{- Direct import of types and functions from the sndio c library. -}

{-# LANGUAGE ForeignFunctionInterface #-}

module SoundStream.CSndio where

import Foreign.C.String (CString)
import Foreign.C.Types
import Foreign.Storable (Storable(..))
import Foreign.Ptr (Ptr)

#include <sndio.h>


-- Types

data SioHdl

data SioPar =
  SioPar
  { bits, bps, sig, le, msb, rchan, pchan, rate, appbufsz, bufsz, round, xrun :: CUInt
  }

-- https://wiki.haskell.org/Foreign_Function_Interface#hsc2hs
-- https://wiki.haskell.org/FFI_cook_book#Working_with_structs

instance Storable SioPar where
  sizeOf _ = #size struct sio_par
  alignment _ = #alignment struct sio_par
  peek ptr = SioPar
             <$> (#peek struct sio_par, bits) ptr
             <*> (#peek struct sio_par, bps) ptr
             <*> (#peek struct sio_par, sig) ptr
             <*> (#peek struct sio_par, le) ptr
             <*> (#peek struct sio_par, msb) ptr
             <*> (#peek struct sio_par, rchan) ptr
             <*> (#peek struct sio_par, pchan) ptr
             <*> (#peek struct sio_par, rate) ptr
             <*> (#peek struct sio_par, appbufsz) ptr
             <*> (#peek struct sio_par, bufsz) ptr
             <*> (#peek struct sio_par, round) ptr
             <*> (#peek struct sio_par, xrun) ptr
  poke ptr sioPar =
    do (#poke struct sio_par, bits) ptr (bits sioPar)
       (#poke struct sio_par, bps) ptr (bps sioPar)
       (#poke struct sio_par, sig) ptr (sig sioPar)
       (#poke struct sio_par, le) ptr (le sioPar)
       (#poke struct sio_par, msb) ptr (msb sioPar)
       (#poke struct sio_par, rchan) ptr (rchan sioPar)
       (#poke struct sio_par, pchan) ptr (pchan sioPar)
       (#poke struct sio_par, rate) ptr (rate sioPar)
       (#poke struct sio_par, appbufsz) ptr (appbufsz sioPar)
       (#poke struct sio_par, bufsz) ptr (bufsz sioPar)
       (#poke struct sio_par, round) ptr (SoundStream.CSndio.round sioPar)
       (#poke struct sio_par, xrun) ptr (xrun sioPar)


-- Constants

sioDevany :: String
sioDevany = #const_str SIO_DEVANY

sioPlay, sioRec :: CUInt
sioPlay = #const SIO_PLAY
sioRec = #const SIO_REC

sioIgnore, sioSync, sioError :: CUInt
sioIgnore = #const SIO_IGNORE
sioSync = #const SIO_SYNC
sioError = #const SIO_ERROR

sioLeNative :: CUInt
sioLeNative = #const SIO_LE_NATIVE


-- Functions

foreign import ccall "sio_close" sioClose :: Ptr SioHdl -> IO ()
foreign import ccall "sio_getpar" sioGetpar :: Ptr SioHdl -> Ptr SioPar -> IO CInt
foreign import ccall "sio_initpar" sioInitpar :: Ptr SioPar -> IO ()
foreign import ccall "sio_open" sioOpen :: CString -> CUInt -> CInt -> IO (Ptr SioHdl)
foreign import ccall "sio_setpar" sioSetpar :: Ptr SioHdl -> Ptr SioPar -> IO CInt
foreign import ccall "sio_start" sioStart :: Ptr SioHdl -> IO CInt
foreign import ccall "sio_write" sioWrite :: Ptr SioHdl -> Ptr () -> CSize -> IO CSize
