{-

Variant(s) of the discrete Fourier transform (DFT).

-}

module SoundStream.DFT (normalizedHartley) where

import Data.Complex (Complex((:+)), realPart, imagPart)
import Foreign.Storable (Storable)
import qualified SoundStream.Block as Block
import qualified SoundStream.PureFFT as PureFFT

-- The Discrete Hartley Transform, normalized so that it's its own inverse.
-- Input must be nonempty; output will be the same size.
--
-- Formula: for a length-n input,
--   output_k = 1 / sqrt n * sum (j = 0 .. n-1) input_j * (cos (2*pi*j*k/n) + sin (2*pi*j*k/n))
normalizedHartley :: (Block.Block b, RealFloat r, Storable r) => b r -> b r
normalizedHartley input =
  let realAsComplex x = x :+ 0
      scale = 1 / sqrt (fromIntegral (Block.size input))
  in Block.fromList
     [ (realPart x + imagPart x) * scale
     | x <- PureFFT.fft (Block.toList (Block.bmap realAsComplex input))
     ]
