module SoundStream.PureFFT (fft, inverseFft) where

import Data.Complex

-- Input length must be at least 1.
--
-- Only fast when input length is a power of 2 times a small number. For an
-- input of size n2^k, runtime is O(n^2 k 2^k).
--
-- TODO: Make it faster.
fft :: RealFloat a => [Complex a] -> [Complex a]
fft l@[x] = l
fft l@(_ : _ : _) =
  let n = length l
  in if mod n 2 == 0
     then let (even, odd) = sepEvenOdd l [] []
              p = fft even
              q = fft odd
          in zipWith3 (\i x y -> x + y * cis (2 * pi * fromIntegral i / fromIntegral n))
             [0..] (p ++ p) (q ++ q)
     else slowDft n l

-- The (approximate) inverse of fft. Same caveats about input length as for fft applies.
inverseFft :: RealFloat a => [Complex a] -> [Complex a]
inverseFft l =
  let n = length l
  in map (conjugate . (/ fromIntegral n)) (fft (map conjugate l))

sepEvenOdd :: [a] -> [a] -> [a] -> ([a], [a])
sepEvenOdd [] evenAcc oddAcc = (reverse evenAcc, reverse oddAcc)
sepEvenOdd (e : o : t) evenAcc oddAcc = sepEvenOdd t (e : evenAcc) (o : oddAcc)

-- lLength should be the length of l. (To save computation.)
slowDft :: RealFloat a => Int -> [Complex a] -> [Complex a]
slowDft lLength l =
  let rootOfUnity = cis (2 * pi / fromIntegral lLength)
  in [ let a = rootOfUnity ^ j
       in sum [x * a^k | (x, k) <- zip l [0..]]
     | j <- [0 .. lLength - 1]
     ]
