{-

CArrays indexed by Ints where the starting index is 0.

fromGeneralArray and toGeneralArray convert to/from an array type indexed by
Int. fromGeneralArray returns Nothing iff the input isn't zero-indexed.

The Block instance provides additional functionality.

-}

module SoundStream.ZeroBasedCArray (ZeroBasedCArray(toCArray), amap, fromCArray, fromList) where

import qualified Data.Array.IArray as IArray
import qualified Data.Array.CArray as CArray
import Data.Array.CArray (CArray)
import Foreign.Storable (Storable)

newtype ZeroBasedCArray e =
  ZeroBasedCArray { toCArray :: CArray Int e }
  deriving (Eq, Show)

-- Apply a function to each element. Like IArray.amap.
amap :: (Storable e, Storable f) => (e -> f) -> ZeroBasedCArray e -> ZeroBasedCArray f
amap f (ZeroBasedCArray a) = ZeroBasedCArray (IArray.amap f a)

fromCArray :: Storable e => CArray Int e -> Maybe (ZeroBasedCArray e)
fromCArray a =
  let (lower, _) = IArray.bounds a
  in if lower == 0
     then Just (ZeroBasedCArray a)
     else Nothing

fromList :: Storable e => [e] -> ZeroBasedCArray e
fromList elements = ZeroBasedCArray (IArray.listArray (0, length elements - 1) elements)
