{-

Plays a low pulse every second, and a higher pulse at a slightly lower rate, so
that the two pulse sequences go out of sync and then line up again when the
countdown reaches zero.

-}

module SoundStream.CycleTimer (play) where

import SoundStream.BlockUtil (groupListIntoBlocks)
import SoundStream.Duration
import SoundStream.Frequency
import qualified SoundStream.PulseTrack as PulseTrack
import qualified SoundStream.Sndio as Sndio
import SoundStream.ZeroBasedCArray (ZeroBasedCArray)

{-

  Parameters

-}

-- The total length of the countdown, in seconds.
totalTimerDuration :: Int
totalTimerDuration = 32

baseFrequency :: Frequency Double
baseFrequency = fromHz 440

noteDuration :: Duration Double
noteDuration = fromSeconds 0.05

{-

  Implementation

-}

play :: (ZeroBasedCArray Sndio.Sample -> IO ()) -> Sndio.Parameters -> IO ()
play write parameters =
  let tracks = [ makeTrack totalTimerDuration baseFrequency
               , makeTrack (totalTimerDuration - 1) (fromHz (4/3 * toHz baseFrequency))
               ]
      soundStream = map doubleToSample (PulseTrack.render parameters tracks)
                    -- TODO: Figure out why the sound cuts off early if we
                    -- don't append some zeros like so:
                    ++ replicate 10000 0
      soundBlocks = groupListIntoBlocks 1024 soundStream
  in do sequence_ $ map write soundBlocks

-- Plays a total of (numIntervals+2) notes over totalTimerDuration seconds.
makeTrack :: Int -> Frequency Double -> PulseTrack.PulseTrack
makeTrack numIntervals pitch =
  let -- Since notes start sounding before the instant they're scheduled, we start the sound a little early.
      startTime = noteDuration
  in PulseTrack.PulseTrack
     { PulseTrack.amplitude = 0.5
     , PulseTrack.pitch = pitch
     , PulseTrack.duration = noteDuration
     , PulseTrack.centerTimes =
         [ fromSeconds
           ( toSeconds startTime
           + fromIntegral i * fromIntegral totalTimerDuration / fromIntegral numIntervals
           )
         | i <- [0..numIntervals]
         ]
     }

doubleToSample :: Double -> Sndio.Sample
doubleToSample x =
  max minBound $ min maxBound $
  round
  ( fromIntegral (maxBound :: Sndio.Sample) * x
  )
